/**********************************************************************
// 
//  PIC16F876.
// Mplab 8.91    8 v.1.2
**********************************************************************/
/**********************************************************************
      10-   
.
                     PORTC
                    bgch afed          PORTC
  0 - a,b,c,d,e,f   0101 0000         50
  1- b,c            0101 1111         5F
  2 - a,b,g,e,d     0011 0100         34
  3 - a,b,c,d,g     0001 0110         16
  4 - b,c,f,g       0001 1011        1B
  5 - a,c,d,f,g     1001 0010        92
  6 - a,d,c,e,f,g   1001 0000         90
  7 - a,b,c         0101 0111         57
  8 - a,b,c,d,e,f,g 0001 0000        10
  9 - a,b,c,d,f,g   0001 0010         12
  PORTB
1 - 0x01
2-  0x08
3 - 0x10
4 - 0x02
5 - 0x80
6 - 0x40
7 - 0x20
8 - 0x04
 F = F/8.
   1    8
 n_tmr1 = F/(65536 * 8)     
 Tmr1_in = 65536 - (F/(n_tmr1 * 8))   1.
        ,  Tmr1_in   .

**********************************************************************/
#include <htc.h>
#define _XTAL_FREQ  10000000

__CONFIG( WDTE_OFF& FOSC_HS& CP_OFF& BOREN_ON& LVP_OFF& PWRTE_ON );


#define   Freq_Pin     RA4   // 
#define   Count        RA0     // 
#define   F_dop        RA1     //

//-------------------------------------------------------------------------------------------
const unsigned char iDC7[10] = {0x50, 0x5F, 0x34, 0x16, 0x1B, 0x92, 0x90, 0x57, 0x10, 0x12};
void Init_CPU( void );
void interrupt clock_isr( void );

unsigned char Freq[8];
unsigned long Freq_F;
volatile unsigned int counter;
const unsigned int Tmr1_in = 2997; //   10 
unsigned char fdop, Tmr0_old, n_tmr1, Ind;
unsigned char i, Flag, Flag_D;

/* =================================================
  
 ===================================================  */

/* =================================================
   
 ===================================================  */
void interrupt clock_isr( void )
{
     if ( TMR0IF & TMR0IE )
   {
      counter ++;
      TMR0IF = 0;
   }
   
  if ( TMR1IF & TMR1IE )
   {
      n_tmr1--;

      if ( !n_tmr1 )     //   
      {
	      Count = 0;
          TMR1ON = 0;
         Flag = 1;
      }
      else
	  {
	  TMR1 = Tmr1_in;
	  }
      TMR1IF = 0;
      
   }

   if ( TMR2IE && TMR2IF )
   {
      Ind ++;
	  Flag_D = 1;
      TMR2IF = 0;
   }
}
/* =================================================
  
 ===================================================  */
void Init_CPU( void )
{
   OPTION_REG = 0b10100111;           //  0 1:256,   RA4, 
   CMCON = 0b00000111; //  
   ADCON0 = 0;
   ADCON1 = 0b00000110;
   //  
   TRISA = 0b11010000;                // , 6,7,4-.
   TRISB = 0;                         // .
   TRISC = 0;                         // .
   PORTA = 0x03;
   PORTB = 0;
   PORTC = 0;
  //  Timer
   T1CON = 0b00110001;                // 1:8
   T2CON = 0b00001110;            //     . - 16.  - 2.
   INTCON = 0;
   PEIE = 1;
   TMR1IE = 1;                        //    TMR1
   TMR0IE = 1;                        //    TMR0
   TMR2IE = 1;                        //    TMR2
   GIE    = 1;
}
//**************************************************************

void main( void )
{
   n_tmr1 = 5;          // 1 
   TMR1 = Tmr1_in;      // 1 
   TMR0 = 0;
   fdop = 0;
   counter = 0;
   Ind = 0;
   Init_CPU();

   while ( 1 )
   {
      //******************************************************************************
      //   
      //******************************************************************************
     if ( Flag_D ) 
	 {
     if ( Ind > 7 )
      {
         Ind = 0;
      }

     switch ( Ind )
      {
         case 0:
            if ( Freq[7] )
            {
               PORTC = ( iDC7[Freq[7]] );
               PORTB = 0x01;
            }
            else
            {
               PORTB = 0x00;           //   0
            }
            break;

         case 1:
            if ( !Freq[6] && !Freq[7] )
            {
               PORTB = 0x00;
            }
            else
            {
               PORTC = ( iDC7[Freq[6]] & 0xEF )  ;  // 
               PORTB = 0x08;
            }
            break;

         case 2:
            if ( !Freq[5] && !Freq[6] && !Freq[7] )   //   0
            {
               PORTB = 0x00;
            }
            else
            {
               PORTC = ( iDC7[Freq[5]] );
               PORTB = 0x10;
            }
            break;

         case 3:
            if ( !Freq[4] && !Freq[5] && !Freq[6] && !Freq[7] )  //   0
            {
               PORTB = 0x00;
            }
            else
            {
               PORTC = ( iDC7[Freq[4]] );
               PORTB = 0x02;
            }
            break;

         case 4:
            PORTC = ( iDC7[Freq[3]] & 0xEF ); // 
            PORTB = 0x80;
            break;

         case 5:
            PORTC = ( iDC7[Freq[2]] );
            PORTB = 0x40;
            break;

         case 6:
            PORTC = ( iDC7[Freq[1]] );
            PORTB = 0x20;
            break;

         case 7:
            PORTC = ( iDC7[Freq[0]] );
            PORTB = 0x04;
            break;
      } 
	  Flag_D = 0;
	 }


      if ( Flag )
      {
       Tmr0_old = TMR0;
        GIE = 0;
         while ( Tmr0_old == TMR0 )
         {
            F_dop = 0;
            asm( "nop" ) ;
            asm( "nop" ) ;
            F_dop = 1;
            asm( "nop" ) ;
            asm( "nop" ) ;
            fdop ++;
         }

         fdop = ( 256 - fdop );
		  GIE = 1;
        
        Freq_F =(( counter * 65536 ) + (unsigned int)( Tmr0_old << 8 ) + (unsigned char)fdop);
         
         for ( i = 0; i < 8; i++ )
         {
            if ( i == 7 )
            {
               Freq[7] = Freq_F;
            }
            else
            {
               Freq[i] = Freq_F % 10;
               Freq_F /= 10;
            }
         }

        
         TMR0 = 0;
         fdop = 0;
         counter = 0;
         Flag = 0;
         n_tmr1 = 5;
         TMR1 = Tmr1_in;
         TMR1ON = 1;
         Count = 1;
      }
   }
}




